lltrace - Query or control trace messages

Use the lltrace command to turn trace on and off dynamically or to query
trace messages.


Syntax


lltrace -? | -H | -v | [-m {on | off}] {-c -t {now | start_time} -d
duration | -j jobstep}


Flags


-?       Provides a short usage message.

-H       Provides extended help information.

-v       Displays the name of the command, release number, service
         level, service level date, and lowest level of the operating
         system to run this release.

-m {on | off}
         Turns tracing on or off dynamically for the dispatching cycle
         or specified job.

-c       Specifies to trace the dispatching cycle.

-j jobstep
         Traces the job or job step specified. 

         The format of a full LoadLeveler step identifier is
         host.jobid.stepid.          

         where:
          
         *  host is the name of the machine that assigned the step
            identifier.

         *  jobid is the job number assigned to the job when it was
            submitted.

         *  stepid is the job step number assigned to the job step when
            it was submitted.

         When a job identifier is specified, the command action is taken
         for all steps of the job. 

         The job or step identifier can be specified in an abbreviated
         form, jobid or jobid.stepid, when the command is invoked on the
         same machine that assigned the job and step identifiers.

-t {now |  start_time}
         Specifies the start time of a tracing period for the dispatch
         cycle. 

         where: 

         now      Specifies from now on.

          start_time
                  Specifies a start time using the format:
                   [mm/dd[/[cc]yy]] HH:MM[:SS]                   

                  The tracing period can be used to turn on or off the
                  dispatching cycle trace for a period of time or
                  specify that messages in a period of time be included
                  in the lltrace report.

-d duration
         Specifies in minutes, the duration of a tracing period for the
         dispatch cycle.


Description


The lltrace command is used for debugging purposes. Using the -m flag,
this command instructs LoadLeveler to turn trace on or off dynamically
for a specified job step using -j or a period of dispatching cycles
specified using -c. Without the -m flag, it queries and displays trace
information about a job step or a period of the dispatching cycle, if
tracing for it has been enabled.

Tracing the lifecycle of a job step:

You can trace the lifecycle of your own job steps, while an
administrator can trace job steps for all. Before any job life cycle
tracing can occur, the administrator must configure it using
TRACE=D_JOB_LIFECYCLE. Then tracing must be enabled for a specific job
step in order for the lifecycle information for the step to be
displayed.

Job lifecycle tracing for a job step is started in one of the following
ways: 

*  When the job is submitted with the keyword trace=yes in your job
   command file.

*  When the job is submitted with the LOADL_JOB_TRACE environment
   variable set to yes. 

   If the LOADL_JOB_TRACE environment variable is set to yes,
   interactive jobs that do not have job command files can be traced.

*  Dynamically when issuing this command with the -m on option. 

   Tracing the lifecycle of a job step can be turned on or off
   dynamically whether or not tracing for the job step was enabled at
   its submission time.

Reports produced by this command cover only the period when tracing is
turned on. If the Schedd daemon that owns the job is stopped and
restarted, any dynamic changes to tracing will be lost.

When using the -m option, if the requested job step does not exist in
the system, LoadLeveler will treat it as a historical job step and will
not try turning tracing for it on or off dynamically.

When not using the -m option, even if a job step still exists in the
LoadLeveler queues when the command is run, LoadLeveler will only
display the information that has been logged into the back-end. You
might need to run the command more than once to get the complete
information.

Tracing the dispatching cycle:

Only the administrator can trace the dispatching cycle. Tracing must be
enabled for the dispatching cycle for this information to be displayed.
The tracing can be enabled in the configuration by using the
TRACE=D_DISPATCHING_CYCLE keyword or dynamically when needed using the
-m on option of this command. When the dynamic request is issued during
a dispatching cycle, it takes effect at the start of the next one.

A time period can be specified on a dynamic request, so changes to the
tracing occurs only during the time period requested. A dynamic request
is used to change the default trace option during the requested period
of time. For example: 

*  If LoadLeveler was configured to trace the dispatching cycle from its
   start up (using the TRACE=D_JOB_LIFECYCLE flag), an administrator can
   issue the following dynamic request to turn off the dispatching cycle
   trace for a period of time:

   lltrace -m off -c -t now -d 30

   The lltrace -m on command is ignored because the request to trace the
   dispatching cycle has already been made. A dynamic request to turn on
   tracing when it is already on, or turn off, if it is already off, is
   ignored.

*  If LoadLeveler was not configured to trace the dispatching cycle (the
   TRACE keyword does not have the D_DISPATCHING_CYCLE flag), an
   administrator can issue the following command to turn on the
   dispatching cycle trace for a period of time.

   lltrace -m on -c -t 12/31 12:00:00 -d 5 

LoadLeveler will lose any dynamic trace setting after the negotiator
daemon is recycled, restarted, or recovered on an alternate machine.


Examples


1. To turn on the trace of a job step step_id, dynamically, issue: 

   lltrace -m on -j step_id

2. To display trace information for a job step, step_id that has been
   enabled for tracing, issue:

   lltrace -j step_id

   You should receive a response similar to the following: 

   Index:
   ==================================================
   Step cnbladel1n04v1.clusters.com.3.0 state change that was driven by a system event
   ==================================================
   
   Detail:
   ==================================================
   Logged Time    :2011-01-04 10:33:10.636598
   Occurred Time  :2011-01-04 10:33:03.256303
   Event ID       :
   Node           :
   Message Type   :2
   Message        :
           Step cnbladel1n04v1.clusters.com.3.0 state changed IDLE(old)->IDLE(cur)->JOB_PENDING(new).
   Detail         :
           The central manager sent a start order to the job manager for step cnbladel1n04v1.clusters.com.3.0
           Reason:
   Subject        :Step cnbladel1n04v1.clusters.com.3.0 state change that was driven by a system event.
   Reporter       :LoadL_negotiator on cnbladel1n04v1.clusters.com
   Thread ID      :17
   Function Name  :
   Line Number    :0
   Job Step ID    :cnbladel1n04v1.clusters.com.3.0
   Tag 1          :
   Tag 2          :
   Raw Data       :
           Node Type 0 has 1 machines(s)
                   machines: < 0 >
           Node Information
                   taskCount      = 1
                   minInstances   = 1
                   maxInstances   = 1
                   initiatorCount = 1
                   requirements   = ""
                   preferences    = ""
   ==================================================
   Logged Time    :2011-01-04 10:33:10.636625
   Occurred Time  :2011-01-04 10:33:03.266222
   Event ID       :
   Node           :
   Message Type   :2
   Message        :
           Step cnbladel1n04v1.clusters.com.3.0 state changed IDLE(old)->JOB_PENDING(cur)->JOB_STARTING(new).
   Detail         :
           The job manager is starting up the step.
   Subject        :Step cnbladel1n04v1.clusters.com.3.0 state change that was driven by a system event.
   Reporter       :LoadL_negotiator on cnbladel1n04v1.clusters.com
   Thread ID      :33
   Function Name  :
   Line Number    :0
   Job Step ID    :cnbladel1n04v1.clusters.com.3.0
   Tag 1          :
   Tag 2          :
   Raw Data       :
   ==================================================
   Logged Time    :2011-01-04 10:33:10.621412
   Occurred Time  :2011-01-04 10:33:03.402636
   Event ID       :
   Node           :
   Message Type   :2
   Message        :
           Step cnbladel1n04v1.clusters.com.3.0 state changed REJECT_PENDING(old)->IDLE(cur)->JOB_REJECTED(new).
   Detail         :
           Step was rejected by machine cnbladel1n04v1.clusters.com.
           Reason: 
                    REJECT_STEP: Step cnbladel1n04v1.clusters.com.3.0 reject_count=1 rejected for this reason:
           cnbladel1n04v1.clusters.com: Startd could not load the adapter windows.
           Step cnbladel1n04v1.clusters.com.3.0 put on hold because it has been rejected 1 time(s).
   Subject        :Step cnbladel1n04v1.clusters.com.3.0 state change that was driven by a system event.
   Reporter       :LoadL_schedd on cnbladel1n04v1.clusters.com
   Thread ID      :31
   Function Name  :
   Line Number    :0
   Job Step ID    :cnbladel1n04v1.clusters.com.3.0
   Tag 1          :
   Tag 2          :
   Raw Data       :
   ==================================================
   Logged Time    :2011-01-04 10:33:10.636631
   Occurred Time  :2011-01-04 10:33:03.412205
   Event ID       :
   Node           :
   Message Type   :2
   Message        :
           Step cnbladel1n04v1.clusters.com.3.0 state changed JOB_PENDING(old)->JOB_STARTING(cur)->HOLD(new).
   Detail         :
           Step cnbladel1n04v1.clusters.com is put on hold because it has been rejected 1 time(s).
   Subject        :Step cnbladel1n04v1.clusters.com.3.0 state change that was driven by a system event.
   Reporter       :LoadL_negotiator on cnbladel1n04v1.clusters.com
   Thread ID      :36
   Function Name  :
   Line Number    :0
   Job Step ID    :cnbladel1n04v1.clusters.com.3.0
   Tag 1          :
   Tag 2          :
   Raw Data       :
   ==================================================

3. To turn on the dispatching cycle trace for just the next 2 minutes
   dynamically, issue:

   lltrace -m on -c -t now -d 2

4. To display trace information about those historical dispatching
   cycles in the two minutes that started from some time in the past,
   mm/dd/yyyy HH:MM:SS, issue:

   lltrace -c -t mm/dd/yyyy HH:MM:SS -d 2

   You should receive a response similar to the following: 

   Index:
   ==================================================
   ...
   Cycle 4 (Thu Jan  6 11:35:01 2011) initialize resource RASMsg
   Cycle 4 step c54bc1n05.ppd.pok.ibm.com.1.0 schedule process summary
           cycle 4 step c54bc1n05.ppd.pok.ibm.com.1.0 resource check in ideal spaces
           cycle 4 step c54bc1n05.ppd.pok.ibm.com.1.0 schedule NOW Success
   ...
   ==================================================
   Detail:
   ...
   ==================================================
   Logged Time    : 2011-01-06 11:35:02.889170
   Occurred Time  : 2011-01-06 11:35:01.884823
   Event ID       :
   Node           :
   Message Type   : Information
   Message        :
   Detail         :
           Detail of Machine
                   index 0:c54bc1n05.ppd.pok.ibm.com
                   index 1:c54bc1n06.ppd.pok.ibm.com
                   index 2:c54bc1n07.ppd.pok.ibm.com
                   index 3:c54bc1n08.ppd.pok.ibm.com
           Class inventory
                   Class:85ba
                           Avail Node:4
                           Avail initiator:4
                   Class:large
                           Avail Node:4
                           Avail initiator:4
                   Class:misc
                           Avail Node:4
                           Avail initiator:4
                   Class:parallel
                           Avail Node:4
                           Avail initiator:8
                   Class:small
                           Avail Node:4
                           Avail initiator:4
           Summary of machine resources
                   Real Space
                           available idle machines:4
                   V0   Space
                           available idle machines:4
                   Machines that have free initiators:4
                   Machines that have free dstg initiators:0
   Subject        : Cycle 4 (Thu Jan  6 11:35:01 2011) initialize resource RASMsg
   Reporter       :
   Thread ID      : 44
   Function Name  : virtual void Hard::do_dispatching()
   Line Number    : 122
   Job Step ID    :
   Tag 1          :
   Tag 2          :
   Raw Data       :
   ==================================================
   Logged Time    : 2011-01-06 11:35:02.889340
   Occurred Time  : 2011-01-06 11:35:01.886004
   Event ID       :
   Node           :
   Message Type   : Information
   Message        :
   Detail         :
   Subject        : cycle 4 step c54bc1n05.ppd.pok.ibm.com.1.0 schedule process summary
   Reporter       :
   Thread ID      : 44
   Function Name  : void Hard::do_dispatching_schedule_step(cmStep*, int&, time_t&, cmReservation*)
   Line Number    : 1606
   Job Step ID    : c54bc1n05.ppd.pok.ibm.com.1.0
   Tag 1          :
   Tag 2          :
   Raw Data       :
           cycle 4 step c54bc1n05.ppd.pok.ibm.com.1.0 resource check in ideal spaces
           cycle 4 step c54bc1n05.ppd.pok.ibm.com.1.0 schedule NOW Success
   ==================================================
   Logged Time    : 2011-01-06 11:35:02.889220
   Occurred Time  : 2011-01-06 11:35:01.886068
   Event ID       :
   Node           :
   Message Type   : Information
   Message        :
   Detail         :
           Node 0 Machines < 0-3 >
   Subject        : cycle 4 step c54bc1n05.ppd.pok.ibm.com.1.0 resource check in ideal spaces
   Reporter       :
   Thread ID      : 44
   Function Name  : void Scheduler::create_vector_map(cmStep*, int)
   Line Number    : 2805
   Job Step ID    : c54bc1n05.ppd.pok.ibm.com.1.0
   Tag 1          :
   Tag 2          :
   Raw Data       :
           Minimum initiators needed: 1 per machine, 1 total.
           4 machines can run at least 1 tasks per machine, 4 tasks total.
   ==================================================
   Logged Time    : 2011-01-06 11:35:02.889260
   Occurred Time  : 2011-01-06 11:35:01.886574
   Event ID       :
   Node           :
   Message Type   : Information
   Message        :
   Detail         :
   Subject        : cycle 4 step c54bc1n05.ppd.pok.ibm.com.1.0 schedule NOW Success
   Reporter       :
   Thread ID      : 44
   Function Name  : Accumulator* Hard::schedule_step(cmStep*, int)
   Line Number    : 11202
   Job Step ID    : c54bc1n05.ppd.pok.ibm.com.1.0
   Tag 1          :
   Tag 2          :
   Raw Data       :
           schedule_by_nodes
           Result for stepc54bc1n05.ppd.pok.ibm.com.1.0
           task_count = 1
           Node Type 0 has 1 machine(s)
                   machines: < 0 > 
           Node Information
                   taskCount      = 1
                   minInstances   = 1
                   maxInstances   = 1
                   initiatorCount = 1
                   requirements   = ""
                   preferences    = ""
   ==================================================
   ...


Security


LoadLeveler administrators and users can issue this command.